// Copyright (C) 2004-2022, Stefan Kanthak <stefan.kanthak@nexgo.de>

// https://msdn.microsoft.com/en-us/library/4141z1cx.aspx

#pragma once

#ifndef __APPHELP_H__
#define __APPHELP_H__

typedef	ULONGLONG	QWORD;

// https://msdn.microsoft.com/en-us/library/bb432182.aspx

typedef WORD	TAG;

// https://msdn.microsoft.com/en-us/library/bb432490.aspx

#define TAG_TYPE_NULL		0x1000U	// no value associated with the TAG
#define TAG_TYPE_BYTE		0x2000U	// BYTE value
#define TAG_TYPE_WORD		0x3000U	// WORD value
#define TAG_TYPE_DWORD		0x4000U	// DWORD value
#define TAG_TYPE_QWORD		0x5000U	// ULONGLONG value
#define TAG_TYPE_STRINGREF	0x6000U	// tokenized string value
#define TAG_TYPE_LIST		0x7000U	// list of TAG values
#define TAG_TYPE_STRING		0x8000U	// UNICODE string value
#define TAG_TYPE_BINARY		0x9000U	// binary value
#define TAG_TYPE_MASK		0xF000U

// https://msdn.microsoft.com/en-us/library/bb432489.aspx

typedef DWORD	TAGREF;

#define TAGREF_NULL	0UL
#define TAGREF_ROOT	0UL

// https://msdn.microsoft.com/en-us/library/bb432488.aspx

typedef DWORD	TAGID;

#define TAGID_NULL	0UL
#define TAGID_ROOT	0UL

// https://msdn.microsoft.com/en-us/library/bb432487.aspx

#define TAG_INCLUDE					(TAG_TYPE_NULL | 0x0001U)	// include list entry
#define TAG_GENERAL					(TAG_TYPE_NULL | 0x0002U)	// general purpose shim entry
#define TAG_MATCH_LOGIC_NOT				(TAG_TYPE_NULL | 0x0003U)	// NOT of matching logic entry
#define TAG_APPLY_ALL_SHIMS				(TAG_TYPE_NULL | 0x0004U)	// unused
#define TAG_USE_SERVICE_PACK_FILES			(TAG_TYPE_NULL | 0x0005U)	// service pack information for AppHelp entries
#define TAG_MITIGATION_OS				(TAG_TYPE_NULL | 0x0006U)	// mitigation at operating system scope entry
#define TAG_TRACE_PCA					(TAG_TYPE_NULL | 0x0007U)
#define TAG_INCLUDEEXCLUDEDLL				(TAG_TYPE_NULL | 0x0008U)	// DLL include/exclude entry
#define TAG_RAC_EVENT_OFF				(TAG_TYPE_NULL | 0x0009U)
#define TAG_TELEMETRY_OFF				(TAG_TYPE_NULL | 0x000AU)
#define TAG_SHIM_ENGINE_OFF				(TAG_TYPE_NULL | 0x000BU)
#define TAG_LAYER_PROPAGATION_OFF			(TAG_TYPE_NULL | 0x000CU)
#define TAG_FORCE_CACHE					(TAG_TYPE_NULL | 0x000DU)
#define TAG_MONITORING_OFF				(TAG_TYPE_NULL | 0x000EU)
#define TAG_QUIRK_OFF					(TAG_TYPE_NULL | 0x000FU)
#define TAG_ELEVATED_PROP_OFF				(TAG_TYPE_NULL | 0x0010U)
#define TAG_UPGRADE_ACTION_BLOCK_WEBSETUP		(TAG_TYPE_NULL | 0x0011U)
#define TAG_UPGRADE_ACTION_PROCEED_TO_MEDIASETUP	(TAG_TYPE_NULL | 0x0012U)

#define TAG_MATCH_MODE					(TAG_TYPE_WORD | 0x0001U)	// match mode attribute
#define TAG_QUIRK_COMPONENT_CODE_ID			(TAG_TYPE_WORD | 0x0002U)
#define TAG_QUIRK_CODE_ID				(TAG_TYPE_WORD | 0x0003U)
#define TAG_TAG						(TAG_TYPE_WORD | 0x0801U)	// TAG entry
#define TAG_INDEX_TAG					(TAG_TYPE_WORD | 0x0802U)	// index TAG attribute for an index entry
#define TAG_INDEX_KEY					(TAG_TYPE_WORD | 0x0803U)	// index key attribute for an index entry

#define TAG_SIZE					(TAG_TYPE_DWORD | 0x0001U)	// file size attribute
#define TAG_OFFSET					(TAG_TYPE_DWORD | 0x0002U)	// unused
#define TAG_CHECKSUM					(TAG_TYPE_DWORD | 0x0003U)	// file checksum attribute
#define TAG_SHIM_TAGID					(TAG_TYPE_DWORD | 0x0004U)	// shim TAGID attribute
#define TAG_PATCH_TAGID					(TAG_TYPE_DWORD | 0x0005U)	// patch TAGID attribute
#define TAG_MODULE_TYPE					(TAG_TYPE_DWORD | 0x0006U)	// module type attribute
#define TAG_VERDATEHI					(TAG_TYPE_DWORD | 0x0007U)	// high-order portion of the file version date attribute
#define TAG_VERDATELO					(TAG_TYPE_DWORD | 0x0008U)	// low-order portion of the file version date attribute
#define TAG_VERFILEOS					(TAG_TYPE_DWORD | 0x0009U)	// operating system file version attribute
#define TAG_VERFILETYPE					(TAG_TYPE_DWORD | 0x000AU)	// file type attribute
#define TAG_PE_CHECKSUM					(TAG_TYPE_DWORD | 0x000BU)	// PE file checksum attribute
#define TAG_PREVOSMAJORVER				(TAG_TYPE_DWORD | 0x000CU)	// major operating system version attribute
#define TAG_PREVOSMINORVER				(TAG_TYPE_DWORD | 0x000DU)	// minor operating system version attribute
#define TAG_PREVOSPLATFORMID				(TAG_TYPE_DWORD | 0x000EU)	// operating system platform identifier attribute
#define TAG_PREVOSBUILDNO				(TAG_TYPE_DWORD | 0x000FU)	// operating system build number attribute
#define TAG_PROBLEMSEVERITY				(TAG_TYPE_DWORD | 0x0010U)	// block attribute of an AppHelp entry; determines whether the application is hard or soft blocked
#define TAG_LANGID					(TAG_TYPE_DWORD | 0x0011U)	// language identifier of an AppHelp entry
#define TAG_VER_LANGUAGE				(TAG_TYPE_DWORD | 0x0012U)	// language version attribute of a file
#define TAG_UNKNOWN					(TAG_TYPE_DWORD | 0x0013U)
#define TAG_ENGINE					(TAG_TYPE_DWORD | 0x0014U)	// unused
#define TAG_HTMLHELPID					(TAG_TYPE_DWORD | 0x0015U)	// help identifier attribute for an AppHelp entry
#define TAG_INDEX_FLAGS					(TAG_TYPE_DWORD | 0x0016U)	// flags attribute for an index entry
#define TAG_FLAGS					(TAG_TYPE_DWORD | 0x0017U)	// flags attribute for an AppHelp entry
#define TAG_DATA_VALUETYPE				(TAG_TYPE_DWORD | 0x0018U)	// data type attribute for a data entry
#define TAG_DATA_DWORD					(TAG_TYPE_DWORD | 0x0019U)	// DWORD value attribute for a data entry
#define TAG_LAYER_TAGID					(TAG_TYPE_DWORD | 0x001AU)	// layer shim TAGID attribute
#define TAG_MSI_TRANSFORM_TAGID				(TAG_TYPE_DWORD | 0x001BU)	// MSI transform TAGID attribute
#define TAG_LINKER_VERSION				(TAG_TYPE_DWORD | 0x001CU)	// linker version attribute of a file
#define TAG_LINK_DATE					(TAG_TYPE_DWORD | 0x001DU)	// link date attribute of a file
#define TAG_UPTO_LINK_DATE				(TAG_TYPE_DWORD | 0x001EU)	// link date attribute of a file; matching is done up to and including this link date
#define TAG_OS_SERVICE_PACK				(TAG_TYPE_DWORD | 0x001FU)	// operating system service pack attribute for an executable entry
#define TAG_FLAG_TAGID					(TAG_TYPE_DWORD | 0x0020U)	// flags TAGID attribute
#define TAG_RUNTIME_PLATFORM				(TAG_TYPE_DWORD | 0x0021U)	// run-time platform attribute of a file
#define TAG_OS_SKU					(TAG_TYPE_DWORD | 0x0022U)	// operating system SKU attribute for an executable entry
#define TAG_OS_PLATFORM					(TAG_TYPE_DWORD | 0x0023U)	// operating system platform attribute
#define TAG_APP_NAME_RC_ID				(TAG_TYPE_DWORD | 0x0024U)	// application name resource identifier attribute for AppHelp entries
#define TAG_VENDOR_NAME_RC_ID				(TAG_TYPE_DWORD | 0x0025U)	// vendor name resource identifier attribute for AppHelp entries
#define TAG_SUMMARY_MSG_RC_ID				(TAG_TYPE_DWORD | 0x0026U)	// summary message resource identifier attribute for AppHelp entries
#define TAG_VISTA_SKU					(TAG_TYPE_DWORD | 0x0027U)	// Windows Vista SKU attribute
#define TAG_DESCRIPTION_RC_ID				(TAG_TYPE_DWORD | 0x0028U)	// description resource identifier attribute for AppHelp entries
#define TAG_PARAMETER1_RC_ID				(TAG_TYPE_DWORD | 0x0029U)	// parameter1 resource identifier attribute for AppHelp entries
#define TAG_CONTEXT_TAGID				(TAG_TYPE_DWORD | 0x0030U)
#define TAG_EXE_WRAPPER					(TAG_TYPE_DWORD | 0x0031U)
#define TAG_EXE_TYPE					(TAG_TYPE_DWORD | 0x0032U)
#define TAG_FROM_LINK_DATE				(TAG_TYPE_DWORD | 0x0033U)
#define TAG_REVISION_EQ					(TAG_TYPE_DWORD | 0x0034U)
#define TAG_REVISION_LE					(TAG_TYPE_DWORD | 0x0035U)
#define TAG_REVISION_GE					(TAG_TYPE_DWORD | 0x0036U)
#define TAG_DATE_EQ					(TAG_TYPE_DWORD | 0x0037U)
#define TAG_DATE_LE					(TAG_TYPE_DWORD | 0x0038U)
#define TAG_DATE_GE					(TAG_TYPE_DWORD | 0x0039U)
#define TAG_CPU_MODEL_EQ				(TAG_TYPE_DWORD | 0x003AU)
#define TAG_CPU_MODEL_LE				(TAG_TYPE_DWORD | 0x003BU)
#define TAG_CPU_MODEL_GE				(TAG_TYPE_DWORD | 0x003CU)
#define TAG_CPU_FAMILY_EQ				(TAG_TYPE_DWORD | 0x003DU)
#define TAG_CPU_FAMILY_LE				(TAG_TYPE_DWORD | 0x003EU)
#define TAG_CPU_FAMILY_GE				(TAG_TYPE_DWORD | 0x003FU)
#define TAG_CREATOR_REVISION_EQ				(TAG_TYPE_DWORD | 0x0040U)
#define TAG_CREATOR_REVISION_LE				(TAG_TYPE_DWORD | 0x0041U)
#define TAG_CREATOR_REVISION_GE				(TAG_TYPE_DWORD | 0x0042U)
#define TAG_SIZE_OF_IMAGE				(TAG_TYPE_DWORD | 0x0043U)
#define TAG_SHIM_CLASS					(TAG_TYPE_DWORD | 0x0044U)
#define TAG_PACKAGEID_ARCHITECTURE			(TAG_TYPE_DWORD | 0x0045U)
#define TAG_REINSTALL_UPGRADE_TYPE			(TAG_TYPE_DWORD | 0x0046U)
#define TAG_BLOCK_UPGRADE_TYPE				(TAG_TYPE_DWORD | 0x0047U)
#define TAG_ROUTING_MODE				(TAG_TYPE_DWORD | 0x0048U)
#define TAG_OS_VERSION_VALUE				(TAG_TYPE_DWORD | 0x0049U)
#define TAG_CRC_CHECKSUM				(TAG_TYPE_DWORD | 0x004AU)
#define TAG_URL_ID					(TAG_TYPE_DWORD | 0x004BU)
#define TAG_QUIRK_TAGID					(TAG_TYPE_DWORD | 0x004CU)
#define TAG_UNDEFINED					(TAG_TYPE_DWORD | 0x004DU)
#define TAG_MIGRATION_DATA_TYPE				(TAG_TYPE_DWORD | 0x004EU)
#define TAG_UPGRADE_DATA				(TAG_TYPE_DWORD | 0x004FU)
#define TAG_MIGRATION_DATA_TAGID			(TAG_TYPE_DWORD | 0x0050U)
#define TAG_REG_VALUE_TYPE				(TAG_TYPE_DWORD | 0x0051U)
#define TAG_REG_VALUE_DATA_DWORD			(TAG_TYPE_DWORD | 0x0052U)
#define TAG_TEXT_ENCODING				(TAG_TYPE_DWORD | 0x0053U)
#define TAG_UX_BLOCKTYPE_OVERRIDE			(TAG_TYPE_DWORD | 0x0054U)
#define TAG_EDITION					(TAG_TYPE_DWORD | 0x0055U)
#define TAG_FW_LINK_ID					(TAG_TYPE_DWORD | 0x0056U)
#define TAG_KB_ARTICLE_ID				(TAG_TYPE_DWORD | 0x0057U)
#define TAG_UPGRADE_MODE				(TAG_TYPE_DWORD | 0x0058U)
#define TAG_TAGID					(TAG_TYPE_DWORD | 0x0801U)	// TAGID attribute

#define TAG_TIME					(TAG_TYPE_QWORD | 0x0001U)	// time attribute
#define TAG_BIN_FILE_VERSION				(TAG_TYPE_QWORD | 0x0002U)	// bin file version attribute for file entries
#define TAG_BIN_PRODUCT_VERSION				(TAG_TYPE_QWORD | 0x0003U)	// bin product version attribute for file entries
#define TAG_MODTIME					(TAG_TYPE_QWORD | 0x0004U)	// unused
#define TAG_FLAG_MASK_KERNEL				(TAG_TYPE_QWORD | 0x0005U)	// kernel flag mask attribute
#define TAG_UPTO_BIN_PRODUCT_VERSION			(TAG_TYPE_QWORD | 0x0006U)	// bin product version attribute of a file; matching is done up to and including this product version
#define TAG_DATA_QWORD					(TAG_TYPE_QWORD | 0x0007U)	// ULONGLONG value attribute for a data entry
#define TAG_FLAG_MASK_USER				(TAG_TYPE_QWORD | 0x0008U)	// user flag mask attribute
#define TAG_FLAGS_NTVDM1				(TAG_TYPE_QWORD | 0x0009U)	// NTVDM1 flag mask attribute
#define TAG_FLAGS_NTVDM2				(TAG_TYPE_QWORD | 0x000AU)	// NTVDM2 flag mask attribute
#define TAG_FLAGS_NTVDM3				(TAG_TYPE_QWORD | 0x000BU)	// NTVDM3 flag mask attribute
#define TAG_FLAG_MASK_SHELL				(TAG_TYPE_QWORD | 0x000CU)	// shell flag mask attribute
#define TAG_UPTO_BIN_FILE_VERSION			(TAG_TYPE_QWORD | 0x000DU)	// bin file version attribute of a file; matching is done up to and including this file version
#define TAG_FLAG_MASK_FUSION				(TAG_TYPE_QWORD | 0x000EU)	// FUSION flag mask attribute
#define TAG_FLAG_PROCESSPARAM				(TAG_TYPE_QWORD | 0x000FU)	// process param flag attribute
#define TAG_FLAG_LUA					(TAG_TYPE_QWORD | 0x0010U)	// LUA flag attribute
#define TAG_FLAG_INSTALL				(TAG_TYPE_QWORD | 0x0011U)	// install flag attribute
#define TAG_FROM_BIN_PRODUCT_VERSION			(TAG_TYPE_QWORD | 0x0012U)
#define TAG_FROM_BIN_FILE_VERSION			(TAG_TYPE_QWORD | 0x0013U)
#define TAG_PACKAGEID_VERSION				(TAG_TYPE_QWORD | 0x0014U)
#define TAG_FROM_PACKAGEID_VERSION			(TAG_TYPE_QWORD | 0x0015U)
#define TAG_UPTO_PACKAGEID_VERSION			(TAG_TYPE_QWORD | 0x0016U)
#define TAG_OSMAXVERSIONTESTED				(TAG_TYPE_QWORD | 0x0017U)
#define TAG_FROM_OSMAXVERSIONTESTED			(TAG_TYPE_QWORD | 0x0018U)
#define TAG_UPTO_OSMAXVERSIONTESTED			(TAG_TYPE_QWORD | 0x0019U)
#define TAG_FLAG_MASK_WINRT				(TAG_TYPE_QWORD | 0x001AU)
#define TAG_REG_VALUE_DATA_QWORD			(TAG_TYPE_QWORD | 0x001BU)
#define TAG_QUIRK_ENABLED_UPTO_VERSION			(TAG_TYPE_QWORD | 0x001CU)
#define TAG_QUIRK_ENABLED_VERSION_LT			(TAG_TYPE_QWORD | 0x001CU)
#define TAG_SOURCE_OS					(TAG_TYPE_QWORD | 0x001DU)
#define TAG_SOURCE_OS_LTE				(TAG_TYPE_QWORD | 0x001EU)
#define TAG_SOURCE_OS_GTE				(TAG_TYPE_QWORD | 0x001FU)

#define TAG_NAME					(TAG_TYPE_STRINGREF | 0x0001U)	// name attribute
#define TAG_DESCRIPTION					(TAG_TYPE_STRINGREF | 0x0002U)	// description entry
#define TAG_MODULE					(TAG_TYPE_STRINGREF | 0x0003U)	// module attribute
#define TAG_API						(TAG_TYPE_STRINGREF | 0x0004U)	// API entry
#define TAG_VENDOR					(TAG_TYPE_STRINGREF | 0x0005U)	// vendor name attribute
#define TAG_APP_NAME					(TAG_TYPE_STRINGREF | 0x0006U)	// application name attribute that describes an application entry in a shim database
#define TAG_COMMAND_LINE				(TAG_TYPE_STRINGREF | 0x0008U)	// command line attribute that is used when passing arguments to a shim, for example
#define TAG_COMPANY_NAME				(TAG_TYPE_STRINGREF | 0x0009U)	// company name attribute
#define TAG_DLLFILE					(TAG_TYPE_STRINGREF | 0x000AU)	// DLL file attribute for a shim entry
#define TAG_WILDCARD_NAME				(TAG_TYPE_STRINGREF | 0x000BU)	// wildcard name attribute for an executable entry with a wildcard as the file name
#define TAG_PRODUCT_NAME				(TAG_TYPE_STRINGREF | 0x0010U)	// product name attribute
#define TAG_PRODUCT_VERSION				(TAG_TYPE_STRINGREF | 0x0011U)	// product version attribute
#define TAG_FILE_DESCRIPTION				(TAG_TYPE_STRINGREF | 0x0012U)	// file description attribute
#define TAG_FILE_VERSION				(TAG_TYPE_STRINGREF | 0x0013U)	// file version attribute
#define TAG_ORIGINAL_FILENAME				(TAG_TYPE_STRINGREF | 0x0014U)	// original file name attribute
#define TAG_INTERNAL_NAME				(TAG_TYPE_STRINGREF | 0x0015U)	// internal file name attribute
#define TAG_LEGAL_COPYRIGHT				(TAG_TYPE_STRINGREF | 0x0016U)	// copyright attribute
#define TAG_16BIT_DESCRIPTION				(TAG_TYPE_STRINGREF | 0x0017U)	// 16-bit description attribute
#define TAG_APPHELP_DETAILS				(TAG_TYPE_STRINGREF | 0x0018U)	// AppHelp details message information attribute
#define TAG_LINK_URL					(TAG_TYPE_STRINGREF | 0x0019U)	// AppHelp online link URL attribute
#define TAG_LINK_TEXT					(TAG_TYPE_STRINGREF | 0x001AU)	// AppHelp online link text attribute
#define TAG_APPHELP_TITLE				(TAG_TYPE_STRINGREF | 0x001BU)	// AppHelp title attribute
#define TAG_APPHELP_CONTACT				(TAG_TYPE_STRINGREF | 0x001CU)	// AppHelp vendor contact attribute
#define TAG_SXS_MANIFEST				(TAG_TYPE_STRINGREF | 0x001DU)	// side-by-side manifest entry
#define TAG_DATA_STRING					(TAG_TYPE_STRINGREF | 0x001EU)	// string attribute for a data entry
#define TAG_MSI_TRANSFORM_FILE				(TAG_TYPE_STRINGREF | 0x001FU)	// file name attribute of an MSI transformation entry
#define TAG_16BIT_MODULE_NAME				(TAG_TYPE_STRINGREF | 0x0020U)	// 16-bit module name attribute
#define TAG_LAYER_DISPLAYNAME				(TAG_TYPE_STRINGREF | 0x0021U)	// unused
#define TAG_COMPILER_VERSION				(TAG_TYPE_STRINGREF | 0x0022U)	// shim database compiler version
#define TAG_ACTION_TYPE					(TAG_TYPE_STRINGREF | 0x0023U)	// unused
#define TAG_EXPORT_NAME					(TAG_TYPE_STRINGREF | 0x0024U)	// export file name attribute
#define TAG_VENDOR_ID					(TAG_TYPE_STRINGREF | 0x0025U)
#define TAG_DEVICE_ID					(TAG_TYPE_STRINGREF | 0x0026U)
#define TAG_SUB_VENDOR_ID				(TAG_TYPE_STRINGREF | 0x0027U)
#define TAG_SUB_SYSTEM_ID				(TAG_TYPE_STRINGREF | 0x0028U)
#define TAG_PACKAGEID_NAME				(TAG_TYPE_STRINGREF | 0x0029U)
#define TAG_PACKAGEID_PUBLISHER				(TAG_TYPE_STRINGREF | 0x002AU)
#define TAG_PACKAGEID_LANGUAGE				(TAG_TYPE_STRINGREF | 0x002BU)
#define TAG_URL						(TAG_TYPE_STRINGREF | 0x002CU)
#define TAG_MANUFACTURER				(TAG_TYPE_STRINGREF | 0x002DU)
#define TAG_MODEL					(TAG_TYPE_STRINGREF | 0x002EU)
#define TAG_DATE					(TAG_TYPE_STRINGREF | 0x002FU)
#define TAG_REG_VALUE_NAME				(TAG_TYPE_STRINGREF | 0x0030U)
#define TAG_REG_VALUE_DATA_SZ				(TAG_TYPE_STRINGREF | 0x0031U)
#define TAG_MIGRATION_DATA_TEXT				(TAG_TYPE_STRINGREF | 0x0032U)
#define TAG_APP_STORE_PRODUCT_ID			(TAG_TYPE_STRINGREF | 0x0033U)
#define TAG_MORE_INFO_URL				(TAG_TYPE_STRINGREF | 0x0034U)

#define TAG_DATABASE					(TAG_TYPE_LIST | 0x0001U)	// database entry
#define TAG_LIBRARY					(TAG_TYPE_LIST | 0x0002U)	// library entry
#define TAG_INEXCLUDE					(TAG_TYPE_LIST | 0x0003U)	// include and exclude entry
#define TAG_SHIM					(TAG_TYPE_LIST | 0x0004U)	// shim entry that contains the name and purpose information
#define TAG_PATCH					(TAG_TYPE_LIST | 0x0005U)	// patch entry that contains the in-memory patching information
#define TAG_APP						(TAG_TYPE_LIST | 0x0006U)	// application entry
#define TAG_EXE						(TAG_TYPE_LIST | 0x0007U)	// executable entry
#define TAG_MATCHING_FILE				(TAG_TYPE_LIST | 0x0008U)	// matching file entry
#define TAG_SHIM_REF					(TAG_TYPE_LIST | 0x0009U)	// shim definition entry
#define TAG_PATCH_REF					(TAG_TYPE_LIST | 0x000AU)	// patch definition entry
#define TAG_LAYER					(TAG_TYPE_LIST | 0x000BU)	// layer shim entry
#define TAG_FILE					(TAG_TYPE_LIST | 0x000CU)	// file attribute used in a shim entry
#define TAG_APPHELP					(TAG_TYPE_LIST | 0x000DU)	// AppHelp information entry
#define TAG_LINK					(TAG_TYPE_LIST | 0x000EU)	// AppHelp online link information entry
#define TAG_DATA					(TAG_TYPE_LIST | 0x000FU)	// name-value mapping entry
#define TAG_MSI_TRANSFORM				(TAG_TYPE_LIST | 0x0010U)	// MSI transformation entry
#define TAG_MSI_TRANSFORM_REF				(TAG_TYPE_LIST | 0x0011U)	// MSI transformation definition entry
#define TAG_MSI_PACKAGE					(TAG_TYPE_LIST | 0x0012U)	// MSI package entry
#define TAG_FLAG					(TAG_TYPE_LIST | 0x0013U)	// flag entry
#define TAG_MSI_CUSTOM_ACTION				(TAG_TYPE_LIST | 0x0014U)	// MSI custom action entry
#define TAG_FLAG_REF					(TAG_TYPE_LIST | 0x0015U)	// flag definition entry
#define TAG_ACTION					(TAG_TYPE_LIST | 0x0016U)	// unused
#define TAG_LOOKUP					(TAG_TYPE_LIST | 0x0017U)	// lookup entry used for lookup in a driver database
#define TAG_CONTEXT					(TAG_TYPE_LIST | 0x0018U)
#define TAG_CONTEXT_REF					(TAG_TYPE_LIST | 0x0019U)
#define TAG_KDEVICE					(TAG_TYPE_LIST | 0x001AU)
#define TAG_KDRIVER					(TAG_TYPE_LIST | 0x001CU)
#define TAG_MATCHING_DEVICE				(TAG_TYPE_LIST | 0x001EU)
#define TAG_ACPI					(TAG_TYPE_LIST | 0x001FU)
#define TAG_BIOS					(TAG_TYPE_LIST | 0x0020U)
#define TAG_CPU						(TAG_TYPE_LIST | 0x0021U)
#define TAG_OEM						(TAG_TYPE_LIST | 0x0022U)
#define TAG_KFLAG					(TAG_TYPE_LIST | 0x0023U)
#define TAG_KFLAG_REF					(TAG_TYPE_LIST | 0x0024U)
#define TAG_KSHIM					(TAG_TYPE_LIST | 0x0025U)
#define TAG_KSHIM_REF					(TAG_TYPE_LIST | 0x0026U)
#define TAG_REINSTALL_UPGRADE				(TAG_TYPE_LIST | 0x0027U)
#define TAG_KDATA					(TAG_TYPE_LIST | 0x0028U)
#define TAG_BLOCK_UPGRADE				(TAG_TYPE_LIST | 0x0029U)
#define TAG_SPC						(TAG_TYPE_LIST | 0x002AU)
#define TAG_QUIRK					(TAG_TYPE_LIST | 0x002BU)
#define TAG_QUIRK_REF					(TAG_TYPE_LIST | 0x002CU)
#define TAG_BIOS_BLOCK					(TAG_TYPE_LIST | 0x002DU)
#define TAG_MATCHING_INFO_BLOCK				(TAG_TYPE_LIST | 0x002EU)
#define TAG_DEVICE_BLOCK				(TAG_TYPE_LIST | 0x002FU)
#define TAG_MIGRATION_DATA				(TAG_TYPE_LIST | 0x0030U)
#define TAG_MIGRATION_DATA_REF				(TAG_TYPE_LIST | 0x0031U)
#define TAG_MATCHING_REG				(TAG_TYPE_LIST | 0x0032U)
#define TAG_MATCHING_TEXT				(TAG_TYPE_LIST | 0x0033U)
#define TAG_MACHINE_BLOCK				(TAG_TYPE_LIST | 0x0034U)
#define TAG_OS_UPGRADE					(TAG_TYPE_LIST | 0x0035U)
#define TAG_PACKAGE					(TAG_TYPE_LIST | 0x0036U)
#define TAG_PICK_ONE					(TAG_TYPE_LIST | 0x0037U)
#define TAG_MATCH_PLUGIN				(TAG_TYPE_LIST | 0x0038U)
#define TAG_MIGRATION_SHIM				(TAG_TYPE_LIST | 0x0039U)
#define TAG_UPGRADE_DRIVER_BLOCK			(TAG_TYPE_LIST | 0x003AU)
#define TAG_MIGRATION_SHIM_REF				(TAG_TYPE_LIST | 0x003CU)
#define TAG_CONTAINS_FILE				(TAG_TYPE_LIST | 0x003DU)
#define TAG_CONTAINS_HWID				(TAG_TYPE_LIST | 0x003EU)
#define TAG_DRIVER_PACKAGE_BLOCK			(TAG_TYPE_LIST | 0x003FU)
#define TAG_STRINGTABLE					(TAG_TYPE_LIST | 0x0801U)	// string table entry
#define TAG_INDEXES					(TAG_TYPE_LIST | 0x0802U)	// indexes entry that defines all the indexes in a shim database
#define TAG_INDEX					(TAG_TYPE_LIST | 0x0803U)	// index entry that defines an index in a shim database

#define TAG_STRINGTABLE_ITEM				(TAG_TYPE_STRING | 0x0801U)	// string table item entry

#define TAG_PATCH_BITS					(TAG_TYPE_BINARY | 0x0002U)	// patch file bits attribute
#define TAG_FILE_BITS					(TAG_TYPE_BINARY | 0x0003U)	// file bits attribute
#define TAG_EXE_ID					(TAG_TYPE_BINARY | 0x0004U)	// GUID attribute of an executable entry
#define TAG_DATA_BITS					(TAG_TYPE_BINARY | 0x0005U)	// data bits attribute
#define TAG_MSI_PACKAGE_ID				(TAG_TYPE_BINARY | 0x0006U)	// MSI package identifier attribute of an MSI package
#define TAG_DATABASE_ID					(TAG_TYPE_BINARY | 0x0007U)	// GUID attribute of a database
#define TAG_CONTEXT_PLATFORM_ID				(TAG_TYPE_BINARY | 0x0008U)
#define TAG_CONTEXT_BRANCH_ID				(TAG_TYPE_BINARY | 0x0009U)
#define TAG_FIX_ID					(TAG_TYPE_BINARY | 0x0010U)
#define TAG_APP_ID					(TAG_TYPE_BINARY | 0x0011U)
#define TAG_REG_VALUE_DATA_BINARY			(TAG_TYPE_BINARY | 0x0012U)
#define TAG_TEXT					(TAG_TYPE_BINARY | 0x0013U)
#define TAG_INDEX_BITS					(TAG_TYPE_BINARY | 0x0801U)	// index bits attribute

// https://msdn.microsoft.com/en-us/library/bb432181.aspx

#define APPTYPE_NONE	0UL

typedef	struct	_APPHELP_DATA
{
	DWORD	dwFlags;
	DWORD	dwSeverity;
	DWORD	dwHTMLHelpID;
	LPWSTR	szAppName;
	TAGREF	trExe;
	LPWSTR	szURL;
	LPWSTR	szLink;
	LPWSTR	szAppTitle;
	LPWSTR	szContact;
	LPWSTR	szDetails;
	DWORD	dwData;
	BOOL	bSPEntry;
} APPHELP_DATA, *LPAPPHELP_DATA;

// https://msdn.microsoft.com/en-us/library/bb432183.aspx

#define ATTRIBUTE_AVAILABLE	0x00000001UL	// attribute is available
#define ATTRIBUTE_FAILED	0x00000002UL	// call failed because attribute is not available

typedef	struct	_ATTRINFO
{
	TAG	tAttrID;
	DWORD	dwFlags;
	union
	{
		QWORD	ullAttr;
		DWORD	dwAttr;
		TCHAR	*lpAttr;
	};
} ATTRINFO, *LPATTRINFO;

// https://msdn.microsoft.com/en-us/library/bb432275.aspx

#define SHIMDB_INDEX_UNIQUE_KEY	0x00000001UL

typedef	struct	_FIND_INFO
{
	TAGID	tiIndex;
	TAGID	tiCurrent;
	TAGID	tiEndIndex;
	TAG	tName;
	DWORD	dwIndexRec;
	DWORD	dwFlags;
	QWORD	ullKey;
	union
	{
		LPCWSTR	szName;
		DWORD	dwName;
		GUID	*pguidName;
	};
} FIND_INFO, *LPFIND_INFO;

// https://msdn.microsoft.com/en-us/library/bb432308.aspx

typedef	DWORD	INDEXID;

#define INDEXID_NULL	~0UL

// not documented!

#define PATCH_STOP	0UL
#define PATCH_REPLACE	2UL
#define PATCH_MATCH	4UL

__declspec(deprecated("undocumented interface, use at your own risk"))
typedef	struct	_PATCH_ACTION
{
	DWORD	Opcode;
	DWORD	ActionSize;		// size of structure
	DWORD	PatternSize;
	DWORD	RVA;
	DWORD	Reserved;
	WCHAR	ModuleName[32];
	BYTE	Pattern[ANYSIZE_ARRAY];
} PATCH_ACTION, *LPPATCH_ACTION;

// https://msdn.microsoft.com/en-us/library/bb432389.aspx

typedef	enum	_PATH_TYPE
{
	DOS_PATH,
	NT_PATH
} PATH_TYPE;

typedef	LPVOID	PDB;

__declspec(deprecated("undocumented interface, use at your own risk"))
typedef	struct	_SDB_HDR
{
	DWORD	MajorVersion;
	DWORD	MinorVersion;
	CHAR	Magic[4];		// "sdbf"
} SDB_HDR, *LPSDB_HDR;

// https://msdn.microsoft.com/en-us/library/bb432483.aspx

#define SDB_DATABASE_MAIN		0x80000000UL	// main database; if not present, database is a custom database
#define SDB_DATABASE_SHIM		0x00010000UL	// database contains application entries to be shimmed
#define SDB_DATABASE_MSI		0x00020000UL	// database contains MSI entries
#define SDB_DATABASE_DRIVERS		0x00040000UL	// database contains driver block entries
#define SDB_DATABASE_DETAILS		0x00080000UL	// database contains AppHelp details
#define SDB_DATABASE_SP_DETAILS		0x00100000UL	// database contains SP AppHelp details
#define SDB_DATABASE_RESOURCE		0x00200000UL	// resource DLL contains AppHelp details
#define SDB_DATABASE_TYPE_MASK		0xF02F0000UL	// mask used to extract main database information
#define SDB_DATABASE_MAIN_SHIM		(SDB_DATABASE_MAIN | SDB_DATABASE_SHIM | SDB_DATABASE_MSI)
#define SDB_DATABASE_MAIN_MSI		(SDB_DATABASE_MAIN | SDB_DATABASE_MSI)
#define SDB_DATABASE_MAIN_DRIVERS	(SDB_DATABASE_MAIN | SDB_DATABASE_DRIVERS)
#define SDB_DATABASE_MAIN_DETAILS	(SDB_DATABASE_MAIN | SDB_DATABASE_DETAILS)
#define SDB_DATABASE_MAIN_SP_DETAILS	(SDB_DATABASE_MAIN | SDB_DATABASE_SP_DETAILS)
#define SDB_DATABASE_MAIN_RESOURCE	(SDB_DATABASE_MAIN | SDB_DATABASE_RESOURCE)

// https://msdn.microsoft.com/en-us/library/bb432459.aspx

#define SDB_MAX_EXES	16
#define SDB_MAX_LAYERS	8
#define SDB_MAX_SDBS	16

#define SHIMREG_DISABLE_SHIM	0x00000001UL
#define SHIMREG_DISABLE_APPHELP	0x00000002UL
#define SHIMREG_APPHELP_NOUI	0x00000004UL
#define SHIMREG_APPHELP_CANCEL	0x10000000UL
#define SHIMREG_DISABLE_SXS	0x00000010UL
#define SHIMREG_DISABLE_LAYER	0x00000020UL
#define SHIMREG_DISABLE_DRIVER	0x00000040UL

typedef	struct	_SDBQUERYRESULT
{
	TAGREF	atrExes[SDB_MAX_EXES];
	DWORD	adwExeFlags[SDB_MAX_EXES];
	TAGREF	atrLayers[SDB_MAX_LAYERS];
	DWORD	dwLayerFlags;
	TAGREF	trApphelp;
	DWORD	dwExeCount;
	DWORD	dwLayerCount;
	GUID	guidID;
	DWORD	dwFlags;
	DWORD	dwCustomSDBMap;
	GUID	rgGuidDB[SDB_MAX_SDBS];
} SDBQUERYRESULT, *LPSDBQUERYRESULT;

#if 0
// {11111111-1111-1111-1111-111111111111}
__declspec(deprecated("undocumented interface, use at your own risk"))
const	GUID	GUID_SYSMAIN_SDB = {0x11111111, 0x1111, 0x1111, {0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11}};

// {D8FF6D16-6A3A-468A-8B44-01714DDC49EA}
__declspec(deprecated("undocumented interface, use at your own risk"))
const	GUID	GUID_MSIMAIN_SDB = {0xD8FF6D16, 0x6A3A, 0x468A, {0x8B, 0x44, 0x01, 0x71, 0x4D, 0xDC, 0x49, 0xEA}};

// {F9AB2228-3312-4A73-B6F9-936D70E112EF}
__declspec(deprecated("undocumented interface, use at your own risk"))
const	GUID	GUID_DRVMAIN_SDB = {0xF9AB2228, 0x3312, 0x4A73, {0xB6, 0xF9, 0x93, 0x6D, 0x70, 0xE1, 0x12, 0xEF}};
#endif

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	AllowPermLayer(LPCWSTR lpwszPath);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	ApphelpCheckInstallShieldPackage(LPVOID  lp,
		                                 LPCWSTR lpwszPath);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	ApphelpCheckMsiPackage(LPVOID  lp,
		                       LPCWSTR lpwszPath,
		                       DWORD   dwUnknown1,
		                       DWORD   dwUnknown2);

// https://msdn.microsoft.com/en-us/library/bb432180.aspx

__declspec(dllimport)
BOOL	WINAPI	ApphelpCheckShellObject(REFCLSID ObjectCLSID,
		                        BOOL     bShimIfNecessary,
		                        QWORD    *pullFlags);

#if 0	// NOTE: exported from KERNEL32.DLL
__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	BaseCheckAppcompatCache(LPCWSTR lpwszApplicationName,
		                        HANDLE  FileHandle,
		                        LPCWSTR lpwEnvironment,
		                        LPDWORD dwReason);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
LONG	WINAPI	BaseCleanupAppcompatCache(VOID);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
LONG	WINAPI	BaseCleanupAppcompatCacheSupport(LPVOID);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
VOID	WINAPI	BaseDumpAppcompatCache(VOID);

// https://msdn.microsoft.com/en-us/library/bb432197.aspx

__declspec(dllimport)
BOOL	WINAPI	BaseFlushAppcompatCache(VOID);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	BaseInitAppcompatCache(VOID);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	BaseInitAppcompatCacheSupport(VOID);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
VOID	WINAPI	BaseUpdateAppcompatCache(DWORD, DWORD, DWORD);
#endif

// https://msdn.microsoft.com/en-us/library/bb432431.aspx

__declspec(dllimport)
TAGID	WINAPI	SdbBeginWriteListTag(PDB lpdb,
		                     TAG tTag);

// https://msdn.microsoft.com/en-us/library/cc895520.aspx

__declspec(dllimport)
VOID	WINAPI	SdbCloseDatabase(PDB lpdb);

// https://msdn.microsoft.com/en-us/library/bb432432.aspx

__declspec(dllimport)
VOID	WINAPI	SdbCloseDatabaseWrite(PDB lpdb);

// https://msdn.microsoft.com/en-us/library/bb432433.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbCommitIndexes(PDB lpdb);

// https://msdn.microsoft.com/en-us/library/bb432434.aspx

__declspec(dllimport)
PDB	WINAPI	SdbCreateDatabase(LPCWSTR   lpwszPath,
		                  PATH_TYPE eType);

// https://msdn.microsoft.com/en-us/library/bb432435.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbDeclareIndex(PDB     lpdb,
		                TAG     tWhich,
		                TAG     tKey,
		                DWORD   dwEntries,
		                BOOL    bUniqueKey,
		                INDEXID *lpiiIndex);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SdbDeletePermLayerKeys(LPCWSTR lpwszPath,
		                       BOOL    bMachine);

// https://msdn.microsoft.com/en-us/library/bb432436.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbEndWriteListTag(PDB   lpdb,
		                   TAGID tiList);

// https://msdn.microsoft.com/en-us/library/bb432437.aspx

__declspec(dllimport)
TAGID	WINAPI	SdbFindFirstDWORDIndexedTag(PDB       lpdb,
		                            TAG       tWhich,
		                            TAG       tKey,
		                            DWORD     dwName,
		                            FIND_INFO *lpFindInfo);

// https://msdn.microsoft.com/en-us/library/bb432438.aspx

__declspec(dllimport)
TAGID	WINAPI	SdbFindFirstTag(PDB   lpdb,
		                TAGID tiParent,
		                TAG   tTag);

// https://msdn.microsoft.com/en-us/library/bb432439.aspx

__declspec(dllimport)
TAGID	WINAPI	SdbFindNextTag(PDB   lpdb,
		               TAGID tiParent,
		               TAGID tiPrevious);

// https://msdn.microsoft.com/en-us/library/bb432440.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbFormatAttribute(ATTRINFO *lpAttrInfo,
		                   LPWSTR   lpchBuffer,
		                   DWORD    dwBufferSize);

// https://msdn.microsoft.com/en-us/library/bb432441.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbFreeFileAttributes(ATTRINFO *lpFileAttributes);

typedef	LPVOID	HSDB;

// https://msdn.microsoft.com/en-us/library/bb432442.aspx

__declspec(dllimport)
VOID	WINAPI	SdbGetAppPatchDir(HSDB   hSDB,
		                  LPWSTR lpszAppPatchPath,
		                  DWORD  cchSize);

// https://msdn.microsoft.com/en-us/library/bb432443.aspx

__declspec(dllimport)
LPVOID	WINAPI	SdbGetBinaryTagData(PDB   lpdb,
		                    TAGID tiWhich);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SdbGetDatabaseID(PDB  lpdb,
		                 GUID *lpGuid);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SdbGetDatabaseVersion(LPCWSTR lpwszFileName,
		                      LPDWORD dwMajorversion,
		                      LPDWORD dwMinorversion);

// https://msdn.microsoft.com/en-us/library/bb432445.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbGetFileAttributes(LPCWSTR  lpwszFileName,
		                     ATTRINFO **lpAttrInfo,
		                     LPDWORD  lpdwAttrCount);

// https://msdn.microsoft.com/en-us/library/bb432446.aspx

__declspec(dllimport)
TAGID	WINAPI	SdbGetFirstChild(PDB   lpdb,
		                 TAGID tiParent);

// https://msdn.microsoft.com/en-us/library/bb432447.aspx

__declspec(dllimport)
TAGID	WINAPI	SdbGetIndex(PDB     lpdb,
		            TAG     tWhich,
		            TAG     tKey,
		            LPDWORD lpdwFlags);

// https://msdn.microsoft.com/en-us/library/bb432448.aspx

#define SDBGMEF_IGNORE_ENVIRONMENT	1UL

__declspec(dllimport)
BOOL	WINAPI	SdbGetMatchingExe(HSDB           hSDB,
		                  LPCWSTR        szPath,
		                  LPCWSTR        szModuleName,
		                  LPCWSTR        szEnvironment,
		                  DWORD          dwFlags,
		                  SDBQUERYRESULT *lpQueryResult);

// https://msdn.microsoft.com/en-us/library/bb432449.aspx

__declspec(dllimport)
TAGID	WINAPI	SdbGetNextChild(PDB   lpdb,
		                TAGID tiParent,
		                TAGID tiPrevious);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SdbGetPermLayerKeys(LPCWSTR lpwszPath,
		                    LPWSTR  lpwszLayers,
		                    LPDWORD lpdwBytes,
		                    DWORD   dwFlags);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
DWORD	WINAPI	SdbGetShowDebugInfoOption(VOID);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SdbGetStandardDatabaseGUID(DWORD dwFlags,
		                           GUID  *lpGuid);

// https://msdn.microsoft.com/en-us/library/bb432450.aspx

__declspec(dllimport)
LPWSTR	WINAPI	SdbGetStringTagPtr(PDB   lpdb,
		                   TAGID tiWhich);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
DWORD	WINAPI	SdbGetTagDataSize(PDB   lpdb,
		                  TAGID tiWhich);

// https://msdn.microsoft.com/en-us/library/bb432451.aspx

__declspec(dllimport)
TAG	WINAPI	SdbGetTagFromTagID(PDB   lpdb,
		                   TAGID tiWhich);

// not documented!

#define GRABMI_FILTER_NORMAL		0UL
#define GRABMI_FILTER_PRIVACY		1UL
#define GRABMI_FILTER_DRIVERS		2UL
#define GRABMI_FILTER_VERBOSE		3UL
#define GRABMI_FILTER_SYSTEM		4UL
#define GRABMI_FILTER_THISFILEONLY	5UL
//					0x10000000UL	// dont write closing XML </DATABASE> tag
//					0x20000000UL	// append to output file,
							// dont write XML header and opening <DATABASE> tag
//					0x40000000UL	// get up to 25 matching files only
//					0x80000000UL	// dont examine into subdirectories

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SdbGrabMatchingInfo(LPCWSTR lpwszExe,
		                    DWORD   dwFilter,
		                    LPCWSTR lpwszFile);

__declspec(deprecated("undocumented interface, use at your own risk"))
typedef	BOOL	(WINAPI	Callback) (LPVOID   lpContext,
			           LPCWSTR  lpwszFileName,
			           LPCWSTR  lpwszRelativeFileName,
			           ATTRINFO *lpAttrInfo,
			           LPCWSTR  lpwszXmlTag);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
LONG	WINAPI	SdbGrabMatchingInfoEx(LPCWSTR lpwszExe,
		                      DWORD   dwFilter,
		                      LPCWSTR lpwszFile,
		                      BOOL    (*lpCallback) (LPVOID, LPCWSTR, LPCWSTR, ATTRINFO *, LPCWSTR),
		                      LPVOID  lpContext);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SdbGUIDFromString(LPCWSTR lpwszGuidString,
		                  GUID    *lpGuid);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SdbGUIDToString(GUID   *lpGuid,
		                LPWSTR lpwszGuidString);

// https://msdn.microsoft.com/en-us/library/bb432452.aspx

#define HID_DOS_PATHS		0x00000001UL	// MS-DOS style path
#define HID_DATABASE_FULLPATH	0x00000002UL	// full path
#define HID_NO_DATABASE		0x00000004UL	// lpszDatabasePath parameter is ignored and no database is opened
#define HID_DATABASE_TYPE_MASK	0xF00F0000UL	// parameter specifies a predefined database, lpszDatabasePath parameter is ignored

__declspec(dllimport)
HSDB	WINAPI	SdbInitDatabase(DWORD   dwFlags,
		                LPCWSTR lpszDatabasePath);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SdbIsNullGUID(GUID *lpGuid);

// https://msdn.microsoft.com/en-us/library/bb432453.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbIsStandardDatabase(GUID *lpGuid);

// https://msdn.microsoft.com/en-us/library/bb432454.aspx

__declspec(dllimport)
QWORD	WINAPI	SdbMakeIndexKeyFromString(LPCWSTR lpwszKey);

// https://msdn.microsoft.com/en-us/library/bb432455.aspx

__declspec(dllimport)
PDB	WINAPI	SdbOpenApphelpDetailsDatabase(LPCWSTR lpwszDetailsDatabasePath);

// https://msdn.microsoft.com/en-us/library/bb432456.aspx

__declspec(dllimport)
HMODULE	WINAPI	SdbOpenApphelpResourceFile(LPCWSTR lpwszACResourceFile);

// https://msdn.microsoft.com/en-us/library/bb432457.aspx

__declspec(dllimport)
PDB	WINAPI	SdbOpenDatabase(LPCWSTR   lpwszPath,
		                PATH_TYPE eType);

// https://msdn.microsoft.com/en-us/library/bb432458.aspx

__declspec(dllimport)
DWORD	WINAPI	SdbQueryDataExTagID(PDB     lpdb,
		                    TAGID   tiExe,
		                    LPCWSTR lpszDataName,
		                    LPDWORD lpdwDataType,
		                    LPVOID  lpBuffer,
		                    LPDWORD lpcbBufferSize,
		                    TAGID   *lptiData);

// https://msdn.microsoft.com/en-us/library/bb432460.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbReadApphelpDetailsData(PDB          lpdb,
		                          APPHELP_DATA *lpData);

// https://msdn.microsoft.com/en-us/library/bb432461.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbReadBinaryTag(PDB    lpdb,
		                 TAGID  tiWhich,
		                 LPBYTE lpBuffer,
		                 DWORD  dwBufferSize);

// https://msdn.microsoft.com/en-us/library/bb432462.aspx

__declspec(dllimport)
DWORD	WINAPI	SdbReadDWORDTag(PDB   lpdb,
		                TAGID tiWhich,
		                DWORD dwDefault);

// https://msdn.microsoft.com/en-us/library/bb432463.aspx

__declspec(dllimport)
QWORD	WINAPI	SdbReadQWORDTag(PDB   lpdb,
		                TAGID tiWhich,
		                QWORD qwDefault);

// https://msdn.microsoft.com/en-us/library/bb432464.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbReadStringTag(PDB    lpdb,
		                 TAGID  tiWhich,
		                 LPWSTR lpwszBuffer,
		                 DWORD  cchBufferSize);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
WORD	WINAPI	SdbReadWORDTag(PDB   lpdb,
		               TAGID tiWhich,
		               WORD  wDefault);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SdbRegisterDatabase(LPCWSTR lpszDatabasePath,
		                    DWORD   dwDatabaseType);

// https://msdn.microsoft.com/en-us/library/bb432465.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbRegisterDatabaseEx(LPCWSTR lpszDatabasePath,
		                      DWORD   dwDatabaseType,
		                      QWORD   *lpTimeStamp);

// https://msdn.microsoft.com/en-us/library/cc984963.aspx

__declspec(dllimport)
VOID	WINAPI	SdbReleaseDatabase(HSDB hSDB);

// https://msdn.microsoft.com/en-us/library/bb432466.aspx

__declspec(dllimport)
VOID	WINAPI	SdbReleaseMatchingExe(HSDB   hSDB,
		                      TAGREF trExe);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SdbSetPermLayerKeys(LPCWSTR lpwszPath,
		                    LPCWSTR lpwszLayers,
		                    BOOL    bMachine);

// https://msdn.microsoft.com/en-us/library/bb432467.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbStartIndexing(PDB     lpdb,
		                 INDEXID iiWhich);

// https://msdn.microsoft.com/en-us/library/bb432468.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbStopIndexing(PDB     lpdb,
		                INDEXID iiWhich);

// https://msdn.microsoft.com/en-us/library/bb432469.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbTagRefToTagID(HSDB   hSDB,
		                 TAGREF trWhich,
		                 PDB    *lppdb,
		                 TAGID  *lptiWhich);

// https://msdn.microsoft.com/en-us/library/bb432470.aspx

__declspec(dllimport)
LPCWSTR	WINAPI	SdbTagToString(TAG tag);

// https://msdn.microsoft.com/en-us/library/bb432471.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbUnregisterDatabase(GUID *lpguidDB);

// https://msdn.microsoft.com/en-us/library/bb432472.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbWriteBinaryTag(PDB    lpdb,
		                  TAG    tTag,
		                  LPBYTE lpBuffer,
		                  DWORD  dwSize);

// https://msdn.microsoft.com/en-us/library/bb432473.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbWriteBinaryTagFromFile(PDB     lpdb,
		                          TAG     tTag,
		                          LPCWSTR lpwszPath);

// https://msdn.microsoft.com/en-us/library/bb432474.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbWriteDWORDTag(PDB   lpdb,
		                 TAG   tTag,
		                 DWORD dwData);

// https://msdn.microsoft.com/en-us/library/bb432475.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbWriteNULLTag(PDB lpdb,
		                TAG tTag);

// https://msdn.microsoft.com/en-us/library/bb432476.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbWriteQWORDTag(PDB   lpdb,
		                 TAG   tTag,
		                 QWORD qwData);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SdbWriteStringRefTag(PDB   lpdb,
		                     TAG   tTag,
		                     TAGID tiWhich);

// https://msdn.microsoft.com/en-us/library/bb432477.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbWriteStringTag(PDB     lpdb,
		                  TAG     tTag,
		                  LPCWSTR lpwszData);

// https://msdn.microsoft.com/en-us/library/bb432478.aspx

__declspec(dllimport)
BOOL	WINAPI	SdbWriteWORDTag(PDB  lpdb,
		                TAG  tTag,
		                WORD wData);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	SetPermLayerState(LPCWSTR lpwszPath,
		                  LPCWSTR lpwszLayer,
		                  DWORD   dwFlags,
		                  BOOL    bMachine,
		                  BOOL    bEnable);

__declspec(deprecated("undocumented interface, use at your own risk"))
typedef	enum	_SHIM_LOG_LEVEL
{
	SHIM_ERR  = 1,
	SHIM_WARN = 2,
	SHIM_INFO = 3
} SHIM_LOG_LEVEL;

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	ShimDbgPrint(SHIM_LOG_LEVEL eLevel,
	             LPCSTR         FunctionName,
	             LPCSTR         Format, ...);

__declspec(deprecated("undocumented interface, use at your own risk"))
__declspec(dllimport)
BOOL	WINAPI	ShimDumpCache(HWND      hWindow,
		              HINSTANCE hInstance,
		              LPCSTR    lpszCmdLine,
		              INT       nCmdShow);

// https://msdn.microsoft.com/en-us/library/bb432482.aspx

__declspec(dllimport)
BOOL	WINAPI	ShimFlushCache(HWND      hWindow,
		               HINSTANCE hInstance,
		               LPCSTR    lpszCmdLine,
		               INT       nCmdShow);
#endif // __APPHELP_H__
